@@ -51,7 +51,8 @@ namespace Banshee.IO.Unix
                     throw new Exception ("Failed to allocate native directory string");
                 }
             
-                if (g_mkdir_with_parents (path_ptr, 493 /*0755 - C# doesn't do octal literals*/) == -1) {
+                // 493 == 0755 - C# doesn't do octal literals
+                if (g_mkdir_with_parents (path_ptr, 493) == -1) {
                     Mono.Unix.UnixMarshal.ThrowExceptionForLastError ();
                 }
             } catch (EntryPointNotFoundException) {
