@@ -282,10 +282,7 @@ namespace Banshee.Collection
         
         public virtual StreamPlaybackError PlaybackError {
             get { return playback_error; }
-            set { 
-                playback_error = value;
-                CanPlay &= value == StreamPlaybackError.None;
-            }
+            set { playback_error = value; }
         }
 
         private bool can_save_to_database = true;
@@ -305,8 +302,8 @@ namespace Banshee.Collection
             get { return can_play; }
             set { can_play = value; }
         }
-        
-        private TrackMediaAttributes media_attributes;
+
+        private TrackMediaAttributes media_attributes = TrackMediaAttributes.Default;
         public virtual TrackMediaAttributes MediaAttributes {
             get { return media_attributes; }
             set { media_attributes = value; }
