@@ -52,46 +52,46 @@ namespace Banshee.Library
             Properties.SetString ("RemoveTracksActionLabel", Catalog.GetString ("Remove From Library"));
             AfterInitialized ();
         }
-        
-        protected override void DeleteTrackRange (TrackListDatabaseModel model, RangeCollection.Range range)
-        {
-            // Remove from file system
-            for (int i = range.Start; i <= range.End; i++) {
-                DatabaseTrackInfo track = model [i] as DatabaseTrackInfo;
-                if (track == null)
-                    continue;
 
-                try {
-                    try {
-                        Banshee.IO.Utilities.DeleteFileTrimmingParentDirectories (track.Uri);
-                    } catch (System.IO.FileNotFoundException) {
-                    } catch (System.IO.DirectoryNotFoundException) {
-                    }
-                } catch (Exception e) {
-                    ErrorSource.AddMessage (e.Message, track.Uri.ToString ());
-                }
+        public override string BaseDirectory {
+            get { return Paths.CachedLibraryLocation; }
+        }
+
+        protected override void DeleteTrack (DatabaseTrackInfo track)
+        {
+            try {
+                Banshee.IO.Utilities.DeleteFileTrimmingParentDirectories (track.Uri);
+            } catch (System.IO.FileNotFoundException) {
+            } catch (System.IO.DirectoryNotFoundException) {
             }
-
-            // Remove from database
-            RemoveTrackRange (model, range);
         }
 
-        public override bool AcceptsInputFromSource (Source source)
+        protected override void AddTrack (DatabaseTrackInfo track)
         {
-            return source is IImportSource;
-        }
+            // Ignore if already have it
+            if (track.PrimarySourceId == DbId)
+                return;
 
-        public override void MergeSourceInput (Source source, SourceMergeType mergeType)
-        {
-            if (!(source is IImportSource) || mergeType != SourceMergeType.Source) {
+            // Move it if its from another library source
+            if (track.PrimarySource is LibrarySource) {
+                PrimarySource old_primary = track.PrimarySource;
+                track.PrimarySource = this;
+                track.Save (false);
+                old_primary.NotifyTracksChanged ();
                 return;
             }
-            
-            ((IImportSource)source).Import ();
-        }
-        
-        public override bool CanRename {
-            get { return false; }
+
+            // Otherwise, copy it
+            if (track.PrimarySource is LibrarySource) {
+                // queue in importer or something?
+            }
+
+            /*try {
+                Banshee.IO.Utilities.DeleteFileTrimmingParentDirectories (track.Uri);
+            } catch (System.IO.FileNotFoundException) {
+            } catch (System.IO.DirectoryNotFoundException) {
+            }*/
         }
+
     }
 }
