@@ -119,8 +119,9 @@ namespace Banshee.SmartPlaylist
         }
 
         private string condition_sql;
-        public string ConditionSql {
+        public virtual string ConditionSql {
             get { return condition_sql; }
+            protected set { condition_sql = value; }
         }
 
         private string condition_xml;
@@ -147,14 +148,12 @@ namespace Banshee.SmartPlaylist
             set { limit = value; }
         }
 
-        protected string OrderAndLimit {
-            get {
-                if (IsLimited) {
-                    return String.Format ("{0} {1}", QueryOrder.ToSql (), Limit.ToSql (LimitValue));
-                } else {
-                    return null;
-                }
-            }
+        protected string OrderSql {
+            get { return QueryOrder == null ? null : QueryOrder.ToSql (); }
+        }
+
+        protected string LimitSql {
+            get { return IsLimited ? Limit.ToSql (LimitValue) : null; }
         }
 
         public bool IsLimited {
@@ -176,7 +175,7 @@ namespace Banshee.SmartPlaylist
 
 #region Constructors
 
-        public SmartPlaylistSource (string name, int primarySourceId) : this (null, name, String.Empty, String.Empty, String.Empty, String.Empty, primarySourceId)
+        public SmartPlaylistSource (string name, int primarySourceId) : this (null, name, String.Empty, String.Empty, String.Empty, String.Empty, primarySourceId, 0)
         {
         }
 
@@ -193,7 +192,7 @@ namespace Banshee.SmartPlaylist
         }
 
         // For existing smart playlists that we're loading from the database
-        public SmartPlaylistSource (int? dbid, string name, string condition_xml, string order_by, string limit_number, string limit_criterion, int primarySourceId) :
+        protected SmartPlaylistSource (int? dbid, string name, string condition_xml, string order_by, string limit_number, string limit_criterion, int primarySourceId, int count) :
             base (generic_name, name, dbid, -1, 0, primarySourceId)
         {
             ConditionXml = condition_xml;
@@ -205,6 +204,7 @@ namespace Banshee.SmartPlaylist
             LimitValue.ParseUserQuery (limit_number);
 
             DbId = dbid;
+            SavedCount = count;
 
             InstallProperties ();
 
@@ -287,13 +287,14 @@ namespace Banshee.SmartPlaylist
                         Condition = ?,
                         OrderBy = ?,
                         LimitNumber = ?,
-                        LimitCriterion = ?
+                        LimitCriterion = ?,
+                        CachedCount = ?
                     WHERE SmartPlaylistID = ?",
                 Name, ConditionXml,
                 IsLimited ? QueryOrder.Name : null,
                 IsLimited ? LimitValue.ToSql () : null,
                 IsLimited ? Limit.Name : null,
-                DbId
+                Count, DbId
             ));
             UpdateDependencies ();
         }
@@ -320,15 +321,50 @@ namespace Banshee.SmartPlaylist
         public void Refresh ()
         {
             // Wipe the member list clean and repopulate it 
-            ServiceManager.DbConnection.Execute (String.Format (
+            string reload_str = String.Format (
                 @"DELETE FROM CoreSmartPlaylistEntries WHERE SmartPlaylistID = {0};
                   INSERT INTO CoreSmartPlaylistEntries 
                     SELECT NULL, {0} as SmartPlaylistID, TrackId
                         FROM CoreTracks, CoreArtists, CoreAlbums
-                        WHERE CoreTracks.ArtistID = CoreArtists.ArtistID AND CoreTracks.AlbumID = CoreAlbums.AlbumID AND CoreTracks.PrimarySourceID = {3}
-                        {1} {2}",
-                DbId, PrependCondition("AND"), OrderAndLimit, PrimarySourceId
-            ));
+                        WHERE CoreTracks.ArtistID = CoreArtists.ArtistID AND CoreTracks.AlbumID = CoreAlbums.AlbumID AND CoreTracks.PrimarySourceID = {1}
+                        {2} {3} {4}",
+                DbId, PrimarySourceId, PrependCondition("AND"), OrderSql, LimitSql
+            );
+            ServiceManager.DbConnection.Execute (reload_str);
+
+            // If the smart playlist is limited by file size or media duration, limit it here
+            if (IsLimited && !Limit.RowBased) {
+                // Identify where the cut off mark is
+                HyenaSqliteCommand limit_command = new HyenaSqliteCommand (String.Format (
+                    @"SELECT EntryID, {0} 
+                      FROM CoreTracks, CoreSmartPlaylistEntries
+                      WHERE SmartPlaylistID = {1} AND CoreSmartPlaylistEntries.TrackID = CoreTracks.TrackID
+                      ORDER BY EntryID",
+                    Limit.Column, DbId
+                ));
+
+                long limit = LimitValue.IntValue *  Limit.Factor;
+                long sum = 0;
+                long? cut_off_id = null;
+                using (IDataReader reader = ServiceManager.DbConnection.Query (limit_command)) {
+                    while (reader.Read ()) {
+                        sum += Convert.ToInt64 (reader[1]);
+                        if (sum > limit) {
+                            cut_off_id = Convert.ToInt64 (reader[0]);
+                            break;
+                        }
+                    }
+                }
+
+                // Remove the playlist entries after the cut off
+                if (cut_off_id != null) {
+                    ServiceManager.DbConnection.Execute (new HyenaSqliteCommand (
+                        "DELETE FROM CoreSmartPlaylistEntries WHERE SmartPlaylistID = ? AND EntryID >= ?",
+                        DbId, cut_off_id
+                    ));
+                }
+            }
+
             refreshed = true;
         }
 
@@ -352,7 +388,7 @@ namespace Banshee.SmartPlaylist
             return true;
         }
 
-        public bool CanUnmap {
+        public virtual bool CanUnmap {
             get { return true; }
         }
 
@@ -417,17 +453,19 @@ namespace Banshee.SmartPlaylist
             return String.IsNullOrEmpty (ConditionSql) ? " " : String.Format ("{0} ({1})", with, ConditionSql);
         }
 
-        public static IEnumerable<SmartPlaylistSource> LoadAll ()
+        public static IEnumerable<SmartPlaylistSource> LoadAll (int primary_id)
         {
             using (IDataReader reader = ServiceManager.DbConnection.Query (
-                "SELECT SmartPlaylistID, Name, Condition, OrderBy, LimitNumber, LimitCriterion, PrimarySourceID FROM CoreSmartPlaylists")) {
+                @"SELECT SmartPlaylistID, Name, Condition, OrderBy, LimitNumber, LimitCriterion, PrimarySourceID, CachedCount
+                    FROM CoreSmartPlaylists WHERE PrimarySourceID = ?", primary_id)) {
                 while (reader.Read ()) {
                     SmartPlaylistSource playlist = null;
                     try {
                         playlist = new SmartPlaylistSource (
                             Convert.ToInt32 (reader[0]), reader[1] as string,
                             reader[2] as string, reader[3] as string,
-                            reader[4] as string, reader[5] as string, Convert.ToInt32 (reader[6])
+                            reader[4] as string, reader[5] as string,
+                            Convert.ToInt32 (reader[6]), Convert.ToInt32 (reader[7])
                         );
                     } catch (Exception e) {
                         Log.Warning ("Ignoring Smart Playlist", String.Format ("Caught error: {0}", e), false);
