@@ -35,11 +35,13 @@ using System.Collections.ObjectModel;
 
 using Mono.Unix;
 
+using Hyena;
 using Hyena.Data;
 using Hyena.Query;
 
 using Banshee.Base;
 using Banshee.Collection;
+using Banshee.Configuration;
 using Banshee.ServiceStack;
 
 namespace Banshee.Sources
@@ -58,6 +60,8 @@ namespace Banshee.Sources
         public event EventHandler MessageNotify;
         public event SourceEventHandler ChildSourceAdded;
         public event SourceEventHandler ChildSourceRemoved;
+
+        public delegate void OpenPropertiesDelegate ();
         
         protected Source (string generic_name, string name, int order)
         {
@@ -103,11 +107,15 @@ namespace Banshee.Sources
                 tm_source.TrackModel.Parent = this;
                 ServiceManager.DBusServiceManager.RegisterObject (tm_source.TrackModel);
                 
-                tm_source.ArtistModel.Parent = this;
-                ServiceManager.DBusServiceManager.RegisterObject (tm_source.ArtistModel);
+                if (tm_source.ArtistModel != null) {
+                    tm_source.ArtistModel.Parent = this;
+                    ServiceManager.DBusServiceManager.RegisterObject (tm_source.ArtistModel);
+                }
                 
-                tm_source.AlbumModel.Parent = this;
-                ServiceManager.DBusServiceManager.RegisterObject (tm_source.AlbumModel);
+                if (tm_source.AlbumModel != null) {
+                    tm_source.AlbumModel.Parent = this;
+                    ServiceManager.DBusServiceManager.RegisterObject (tm_source.AlbumModel);
+                }
             }
         }
 
@@ -228,35 +236,39 @@ namespace Banshee.Sources
         
 #region Protected Methods
         
-        protected virtual void SetStatus (string message, bool error)
+        public virtual void SetStatus (string message, bool error)
         {
             SetStatus (message, !error, !error, error ? "dialog-error" : null);
         }
 
-        protected virtual void SetStatus (string message, bool can_close, bool is_spinning, string icon_name)
+        public virtual void SetStatus (string message, bool can_close, bool is_spinning, string icon_name)
         {
-            if (status_message == null) {
-                status_message = new SourceMessage (this);
-                PushMessage (status_message);
+            lock (this) {
+                if (status_message == null) {
+                    status_message = new SourceMessage (this);
+                    PushMessage (status_message);
+                }
+            
+                string status_name = String.Format ("<i>{0}</i>", GLib.Markup.EscapeText (Name));
+                
+                status_message.FreezeNotify ();
+                status_message.Text = String.Format (GLib.Markup.EscapeText (message), status_name);
+                status_message.CanClose = can_close;
+                status_message.IsSpinning = is_spinning;
+                status_message.SetIconName (icon_name);
+                status_message.ClearActions ();
             }
-            
-            string status_name = String.Format ("<i>{0}</i>", GLib.Markup.EscapeText (Name));
-            
-            status_message.FreezeNotify ();
-            status_message.Text = String.Format (GLib.Markup.EscapeText (message), status_name);
-            status_message.CanClose = can_close;
-            status_message.IsSpinning = is_spinning;
-            status_message.SetIconName (icon_name);
-            status_message.ClearActions ();
-            
+                
             status_message.ThawNotify ();
         }
 
-        protected virtual void HideStatus ()
+        public virtual void HideStatus ()
         {
-            if (status_message != null) {
-                RemoveMessage (status_message);
-                status_message = null;
+            lock (this) {
+                if (status_message != null) {
+                    RemoveMessage (status_message);
+                    status_message = null;
+                }
             }
         }
 
@@ -293,6 +305,9 @@ namespace Banshee.Sources
         {
             lock (this) {
                 if (messages.Count > 0) {
+                    foreach (SourceMessage message in messages) {
+                        message.Updated -= HandleMessageUpdated;
+                    }
                     messages.Clear ();
                     OnMessageNotify ();
                 }
@@ -341,6 +356,7 @@ namespace Banshee.Sources
         {
             lock (this) {
                 if (messages.Remove (message)) {
+                    message.Updated -= HandleMessageUpdated;
                     OnMessageNotify ();
                 }
             }   
@@ -386,8 +402,13 @@ namespace Banshee.Sources
                 handler (this, EventArgs.Empty);
             }
         }
-        
-        protected virtual void OnUserNotifyUpdated ()
+
+        public void NotifyUser ()
+        {
+            OnUserNotifyUpdated ();
+        }
+
+        protected void OnUserNotifyUpdated ()
         {
             if (this != ServiceManager.SourceManager.ActiveSource) {
                 EventHandler handler = UserNotifyUpdated;
@@ -494,8 +515,44 @@ namespace Banshee.Sources
         public virtual int Count {
             get { return 0; }
         }
+        
+        public string ConfigurationId {
+            get { return (Parent == null ? UniqueId : Parent.UniqueId).Replace ('.', '_'); }
+        }
 
         public virtual int FilteredCount { get { return Count; } }
+                
+        public virtual string TrackModelPath {
+            get { return null; }
+        }
+        
+#endregion
+
+#region Status Message Stuff        
+        
+        private static DurationStatusFormatters duration_status_formatters = new DurationStatusFormatters ();
+        public static DurationStatusFormatters DurationStatusFormatters {
+            get { return duration_status_formatters; }
+        }
+        
+        protected virtual int StatusFormatsCount {
+            get { return duration_status_formatters.Count; }
+        }
+        
+        protected virtual int CurrentStatusFormat {
+            get { return ConfigurationClient.Get<int> (String.Format ("sources.{0}", ConfigurationId), "status_format", 0); }
+            set { ConfigurationClient.Set<int> (String.Format ("sources.{0}", ConfigurationId), "status_format", value); }
+        }
+        
+        public void CycleStatusFormat ()
+        {
+            int new_status_format = CurrentStatusFormat + 1;
+            if (new_status_format >= StatusFormatsCount) {
+                new_status_format = 0;
+            }
+            
+            CurrentStatusFormat = new_status_format;
+        }
 
         public virtual string GetStatusText ()
         {
@@ -509,20 +566,9 @@ namespace Banshee.Sources
             
             builder.AppendFormat (Catalog.GetPluralString ("{0} item", "{0} items", count), count);
             
-            if (this is IDurationAggregator) {
+            if (this is IDurationAggregator && StatusFormatsCount > 0) {
                 builder.Append (", ");
-
-                TimeSpan span = (this as IDurationAggregator).Duration; 
-                if (span.Days > 0) {
-                    double days = span.Days + (span.Hours / 24.0);
-                    builder.AppendFormat (Catalog.GetPluralString ("{0} day", "{0} days", DoubleToPluralInt (days)), FormatDouble (days));
-                } else if (span.Hours > 0) {
-                    double hours = span.Hours + (span.Minutes / 60.0);
-                    builder.AppendFormat (Catalog.GetPluralString ("{0} hour", "{0} hours", DoubleToPluralInt (hours)), FormatDouble (hours));
-                } else {
-                    double minutes = span.Minutes + (span.Seconds / 60.0);
-                    builder.AppendFormat (Catalog.GetPluralString ("{0} minute", "{0} minutes", DoubleToPluralInt (minutes)), FormatDouble (minutes));
-                }
+                duration_status_formatters[CurrentStatusFormat] (builder, ((IDurationAggregator)this).Duration);
             }
 
             if (this is IFileSizeAggregator) {
@@ -535,25 +581,16 @@ namespace Banshee.Sources
             
             return builder.ToString ();
         }
-
-        private static string FormatDouble (double num)
-        {
-            if (num == (int)num)
-                return Convert.ToString ((int)num);
-            else
-                return String.Format ("{0:0.0}", num);
-        }
         
-        private static int DoubleToPluralInt (double num)
+#endregion
+
+        public override string ToString ()
         {
-            if (num == (int)num)
-                return (int)num;
-            else
-                return (int)num + 1;
+            return Name;
         }
         
         string IService.ServiceName {
-            get { return DBusServiceManager.MakeDBusSafeString (Name) + "Source"; }
+            get { return String.Format ("{0}{1}", DBusServiceManager.MakeDBusSafeString (Name), "Source"); }
         }
         
         IDBusExportable IDBusExportable.Parent {
@@ -565,12 +602,5 @@ namespace Banshee.Sources
                 }
             }
         }
-        
-        public virtual string TrackModelPath {
-            get { return null; }
-        }
-        
-#endregion
-        
     }
 }
