@@ -83,8 +83,10 @@ namespace Banshee.Gui.Widgets
                 artwork_manager = ServiceManager.Get<ArtworkManager> ("ArtworkManager");
             }
             
-            ServiceManager.PlayerEngine.EventChanged += OnPlayerEngineEventChanged;
-            ServiceManager.PlayerEngine.StateChanged += OnPlayerEngineStateChanged;
+            ServiceManager.PlayerEngine.ConnectEvent (OnPlayerEvent, 
+                PlayerEvent.StartOfStream | 
+                PlayerEvent.TrackInfoUpdated | 
+                PlayerEvent.StateChange);
         }
         
         public override void Dispose ()
@@ -93,8 +95,9 @@ namespace Banshee.Gui.Widgets
                 GLib.Source.Remove (idle_timeout_id);
             }
             
-            ServiceManager.PlayerEngine.EventChanged -= OnPlayerEngineEventChanged;
-            ServiceManager.PlayerEngine.StateChanged -= OnPlayerEngineStateChanged;
+            if (ServiceManager.PlayerEngine != null) {
+                ServiceManager.PlayerEngine.DisconnectEvent (OnPlayerEvent);
+            }
             
             stage.Iteration -= OnStageIteration;
             stage = null;
@@ -410,20 +413,18 @@ namespace Banshee.Gui.Widgets
             second_line_layout.Dispose ();
         }
         
-        private void OnPlayerEngineEventChanged (object o, PlayerEngineEventArgs args)
+        private void OnPlayerEvent (PlayerEventArgs args)
         {
-            if (args.Event == PlayerEngineEvent.StartOfStream || args.Event == PlayerEngineEvent.TrackInfoUpdated) {
+            if (args.Event == PlayerEvent.StartOfStream || args.Event == PlayerEvent.TrackInfoUpdated) {
                 LoadCurrentTrack ();
-            }
-        }
-        
-        private void OnPlayerEngineStateChanged (object o, PlayerEngineStateArgs args)
-        {
-            if (args.State == PlayerEngineState.Idle && (incoming_track != null || incoming_pixbuf != null)) {
-                if (idle_timeout_id > 0) {
-                    GLib.Source.Remove (idle_timeout_id);
-                } else {
-                    GLib.Timeout.Add (100, IdleTimeout);
+            } else if (args.Event == PlayerEvent.StateChange && (incoming_track != null || incoming_pixbuf != null)) {
+                PlayerEventStateChangeArgs state = (PlayerEventStateChangeArgs)args;
+                if (state.Current == PlayerState.Idle) {
+                    if (idle_timeout_id > 0) {
+                        GLib.Source.Remove (idle_timeout_id);
+                    } else {
+                        GLib.Timeout.Add (100, IdleTimeout);
+                    }
                 }
             }
         }
@@ -431,7 +432,7 @@ namespace Banshee.Gui.Widgets
         private bool IdleTimeout ()
         {
             if (ServiceManager.PlayerEngine.CurrentTrack == null || 
-                ServiceManager.PlayerEngine.CurrentState == PlayerEngineState.Idle) {
+                ServiceManager.PlayerEngine.CurrentState == PlayerState.Idle) {
                 incoming_track = null;
                 incoming_pixbuf = null;
                 
