@@ -71,7 +71,7 @@ namespace Banshee.Gui
             action_service = actionService;
 
             Add (new ActionEntry [] {
-                new ActionEntry ("NewPlaylistAction", Stock.New,
+                new ActionEntry ("NewPlaylistAction", null,
                     Catalog.GetString ("_New Playlist"), "<control>N",
                     Catalog.GetString ("Create a new empty playlist"), OnNewPlaylist),
 
@@ -87,23 +87,28 @@ namespace Banshee.Gui
                     String.Empty, null, null, OnSourceContextMenu),
 
                 new ActionEntry ("ImportSourceAction", null,
-                    Catalog.GetString ("Import Source"), null,
+                    Catalog.GetString ("Import to Library"), null,
                     Catalog.GetString ("Import source to library"), OnImportSource),
 
-                new ActionEntry ("RenameSourceAction", "gtk-edit", 
+                new ActionEntry ("RenameSourceAction", null,
                     "Rename", "F2", "Rename", OnRenameSource),
 
                 new ActionEntry ("ExportPlaylistAction", null,
                     Catalog.GetString ("Export Playlist..."), null,
                     Catalog.GetString ("Export a playlist"), OnExportPlaylist),
 
-                new ActionEntry ("UnmapSourceAction", Stock.Delete,
+                new ActionEntry ("UnmapSourceAction", null,
                     "Unmap", "<shift>Delete", null, OnUnmapSource),
                     
-                new ActionEntry ("SourcePropertiesAction", Stock.Properties,
+                new ActionEntry ("SourcePropertiesAction", null,
                     "Source Properties", null, null, OnSourceProperties),
             });
 
+            this["NewPlaylistAction"].IconName = Stock.New;
+            this["UnmapSourceAction"].IconName = Stock.Delete;
+            this["RenameSourceAction"].IconName = "gtk-edit";
+            this["SourcePropertiesAction"].IconName = Stock.Properties;
+
             AddImportant (
                 new ActionEntry ("RefreshSmartPlaylistAction", Stock.Refresh,
                     Catalog.GetString ("Refresh"), null,
@@ -160,7 +165,7 @@ namespace Banshee.Gui
             PlaylistSource playlist = new PlaylistSource ("New Playlist", ActivePrimarySource.DbId);
             playlist.Save ();
             playlist.PrimarySource.AddChildSource (playlist);
-            playlist.NotifyUpdated ();
+            playlist.NotifyUser ();
             //SourceView.BeginRenameSource (playlist);
         }
 
@@ -209,7 +214,20 @@ namespace Banshee.Gui
 
             string path = ActionSource.Properties.GetString ("GtkActionPath") ?? "/SourceContextMenu";
             Gtk.Menu menu = action_service.UIManager.GetWidget (path) as Menu;
-            if (menu == null) {
+            if (menu == null || menu.Children.Length == 0) {
+                SourceView.ResetHighlight ();
+                UpdateActions ();
+                return;
+            }
+
+            int visible_children = 0;
+            foreach (Widget child in menu)
+                if (child.Visible)
+                    visible_children++;
+
+            if (visible_children == 0) {
+                SourceView.ResetHighlight ();
+                UpdateActions ();
                 return;
             }
 
@@ -220,9 +238,10 @@ namespace Banshee.Gui
                 UpdateActions ();
             };
         }
-            
+
         private void OnImportSource (object o, EventArgs args)
         {
+            (ActionSource as IImportSource).Import ();
         }
 
         private void OnRenameSource (object o, EventArgs args)
@@ -264,7 +283,7 @@ namespace Banshee.Gui
                 playlist.Save (Banshee.IO.File.OpenWrite (new SafeUri (uri), true), source);
             } catch (Exception e) {
                 Console.WriteLine (e);
-                Log.Error (Catalog.GetString ("Could not export playlist"), e.Message);
+                Log.Error (Catalog.GetString ("Could not export playlist"), e.Message, true);
             }
         }
 
@@ -286,6 +305,11 @@ namespace Banshee.Gui
 
         private void OnSourceProperties (object o, EventArgs args)
         {
+            if (ActionSource.Properties.Contains ("SourceProperties.GuiHandler")) {
+                ActionSource.Properties.Get<Source.OpenPropertiesDelegate> ("SourceProperties.GuiHandler") ();
+                return;
+            }
+
             SmartPlaylistSource source = ActionSource as SmartPlaylistSource;
             if (source != null) {
                 Editor ed = new Editor (source);
@@ -304,10 +328,11 @@ namespace Banshee.Gui
 
             if (source != last_source && source != null) {
                 IUnmapableSource unmapable = source as IUnmapableSource;
+                IImportSource import_source = source as IImportSource;
                 SmartPlaylistSource smart_playlist = source as SmartPlaylistSource;
                 UpdateAction ("UnmapSourceAction", unmapable != null, unmapable != null && unmapable.CanUnmap, source);
                 UpdateAction ("RenameSourceAction", source.CanRename, true, null);
-                UpdateAction ("ImportSourceAction", source is IImportable, true, source);
+                UpdateAction ("ImportSourceAction", import_source != null, import_source != null && import_source.CanImport, source);
                 UpdateAction ("ExportPlaylistAction", source is AbstractPlaylistSource, true, source);
                 UpdateAction ("SourcePropertiesAction", source.HasProperties, true, source);
                 UpdateAction ("RefreshSmartPlaylistAction", smart_playlist != null && smart_playlist.CanRefresh, true, source);
