@@ -31,6 +31,7 @@ using System.Collections.Generic;
 
 using Gtk;
 using Cairo;
+using Mono.Unix;
 
 using Hyena.Gui.Theming;
 using Hyena.Gui.Theatrics;
@@ -68,6 +69,8 @@ namespace Banshee.Sources.Gui
             ConfigureDragAndDrop ();
             RefreshList ();
             ConnectEvents ();
+            
+            RowSeparatorFunc = RowSeparatorHandler;
         }
         
 #region Setup Methods        
@@ -91,7 +94,7 @@ namespace Banshee.Sources.Gui
         
         private void BuildModel ()
         {
-            store = new TreeStore (typeof (Source), typeof (int));
+            store = new TreeStore (typeof (Source), typeof (int), typeof (bool));
             store.SetSortColumnId (1, SortType.Ascending);
             store.ChangeSortColumn ();
             Model = store;
@@ -123,7 +126,7 @@ namespace Banshee.Sources.Gui
                 });
             };
             
-            ServiceManager.PlaybackController.SourceChanged += delegate {
+            ServiceManager.PlaybackController.NextSourceChanged += delegate {
                 QueueDraw ();
             };
             
@@ -278,6 +281,11 @@ namespace Banshee.Sources.Gui
             return false;
         }
         
+        private bool RowSeparatorHandler (TreeModel model, TreeIter iter)
+        {
+            return (bool)store.GetValue (iter, 2);
+        }
+        
 #endregion
 
 #region Source <-> Iter Methods
@@ -300,36 +308,67 @@ namespace Banshee.Sources.Gui
 
         private TreeIter FindSource (Source source)
         {
-            TreeIter iter = TreeIter.Zero;
-            store.GetIterFirst (out iter);
-            return FindSource (source, iter);
+            foreach (TreeIter iter in FindInModel (0, source)) {
+                return iter;
+            }
+            
+            return TreeIter.Zero;
         }
         
-        private TreeIter FindSource (Source source, TreeIter iter)
+        private IEnumerable<TreeIter> FindInModel (int column, object match)
+        {
+            TreeIter iter = TreeIter.Zero;
+            store.GetIterFirst (out iter);
+            return FindInModel (column, match, iter);
+        }
+        
+        private IEnumerable<TreeIter> FindInModel (int column, object match, TreeIter iter)
         {
             if (!store.IterIsValid (iter)) {
-                return TreeIter.Zero;
+                yield break;
             }
             
             do {
-                if ((store.GetValue (iter, 0) as Source) == source) {
-                    return iter;
+                object result = store.GetValue (iter, column);
+                Type result_type = result != null ? result.GetType () : null;
+                if (result_type != null && ((result_type.IsValueType && result.Equals (match)) || result == match)) {
+                    yield return iter;
                 }
                 
                 if (store.IterHasChild (iter)) {
                     TreeIter citer = TreeIter.Zero;
                     store.IterChildren (out citer, iter);
-                    TreeIter result = FindSource (source, citer);
-                    
-                    if (!result.Equals (TreeIter.Zero)) {
-                        return result;
+                    foreach (TreeIter yiter in FindInModel (column, match, citer)) {
+                        if (!yiter.Equals (TreeIter.Zero)) {
+                            yield return yiter;
+                        }
                     }
                 }
             } while (store.IterNext (ref iter));
-            
-            return TreeIter.Zero;
         }
-
+        
+        /*private void AddRowSeparator (int order)
+        {
+            TreeIter iter = store.InsertNode (order);
+            
+            store.SetValue (iter, 0, null);
+            store.SetValue (iter, 1, order);
+            store.SetValue (iter, 2, true);
+        }
+        
+        private void ClearRowSeparators ()
+        {
+            Queue<TreeIter> to_remove = new Queue<TreeIter> ();
+            foreach (TreeIter iter in FindInModel (2, true)) {
+                to_remove.Enqueue (iter);
+            }
+            
+            while (to_remove.Count > 0) {
+                TreeIter iter = to_remove.Dequeue ();
+                store.Remove (ref iter);
+            }
+        }*/
+        
 #endregion
 
 #region Add/Remove Sources / SourceManager interaction
@@ -359,25 +398,22 @@ namespace Banshee.Sources.Gui
             
             store.SetValue (iter, 0, source);
             store.SetValue (iter, 1, source.Order);
+            store.SetValue (iter, 2, false);
 
             lock (source.Children) {
                 foreach (Source child in source.Children) {
                     AddSource (child, iter);
                 }
             }
-
+            
             source.ChildSourceAdded += OnSourceChildSourceAdded; 
             source.ChildSourceRemoved += OnSourceChildSourceRemoved;
             source.UserNotifyUpdated += OnSourceUserNotifyUpdated;
            
-            if (source.Expanded || (source.AutoExpand != null && source.AutoExpand.Value)) {
+            if (source.Expanded || source.AutoExpand == true) {
                 Expand (iter);
-            }
-            
-            if (source.Parent != null) {
-                if (source.Parent.AutoExpand == true) {
-                    Expand (FindSource (source.Parent));
-                }
+            } else if (source.Parent != null && source.Parent.AutoExpand == true) {
+                Expand (FindSource (source.Parent));
             }
             
             UpdateView ();
@@ -560,7 +596,10 @@ namespace Banshee.Sources.Gui
         }
         
         internal Source NewPlaylistSource {
-            get { return new_playlist_source; }
+            get {
+                return new_playlist_source ??
+                    new_playlist_source = new PlaylistSource (Catalog.GetString ("New Playlist"), ServiceManager.SourceManager.MusicLibrary.DbId);
+            }
         }
 
 #endregion        
