@@ -197,15 +197,11 @@ namespace Banshee.Sources.Gui
                 
                 if (Gtk.Drag.GetSourceWidget (context) == this) {
                     DragDropList<Source> sources = selectionData;
-                    if(sources.Count > 0) {
-                        Banshee.Base.ThreadAssist.SpawnFromMain (delegate {
-                            drop_source.MergeSourceInput (sources[0], SourceMergeType.Source);
-                        });
+                    if (sources.Count > 0) {
+                        drop_source.MergeSourceInput (sources[0], SourceMergeType.Source);
                     }
-                } else {   
-                    Banshee.Base.ThreadAssist.SpawnFromMain (delegate {
-                        drop_source.MergeSourceInput (ServiceManager.SourceManager.ActiveSource, SourceMergeType.ModelSelection);
-                    });
+                } else {
+                    drop_source.MergeSourceInput (ServiceManager.SourceManager.ActiveSource, SourceMergeType.ModelSelection);
                 }
                 
                 Gtk.Drag.Finish (context, true, false, time);
