@@ -116,22 +116,14 @@ namespace Banshee.Lastfm.Radio
         public void Dispose ()
         {
             Connection.StateChanged -= HandleConnectionStateChanged;
-            //ClearChildSources ();
+            Connection.Dispose ();
+            actions.Dispose ();
+            
+            actions = null;
+            connection = null;
+            account = null;
         }
 
-        /*public override void ClearChildSources ()
-        {
-            lock (Children) {
-                foreach (StationSource child in Children) {
-                    //if (SourceManager.ContainsSource (child))
-                    //    SourceManager.RemoveSource (child);
-                    child.Dispose ();
-                }
-            }
-
-            base.ClearChildSources ();
-        }*/
-
         /*public override void AddChildSource (ChildSource source)
         {
             base.AddChildSource (source);
@@ -296,13 +288,13 @@ namespace Banshee.Lastfm.Radio
             OnUpdated ();
         }
 
-        protected override void SetStatus (string message, bool error)
+        public override void SetStatus (string message, bool error)
         {
             base.SetStatus (message, error);
             SetStatus (status_message, this, error, ConnectionState.Connected);
         }
 
-        private void SetStatus (string message, bool error, ConnectionState state)
+        public void SetStatus (string message, bool error, ConnectionState state)
         {
             base.SetStatus (message, error);
             SetStatus (status_message, this, error, state);
