@@ -46,7 +46,7 @@ using Banshee.Widgets;
 
 namespace Banshee.NotificationArea 
 {
-    public class NotificationAreaService : IExtensionService, IDisposable
+    public class NotificationAreaService : IExtensionService
     {
         private INotificationAreaBox notif_area;
         private GtkElementsService elements_service;
@@ -130,7 +130,24 @@ namespace Banshee.NotificationArea
             menu.Show ();
             
             for (int i = 0; i < menu.Children.Length; i++) {
-                if (menu.Children[i].Name == "Next") {
+                if (menu.Children[i].Name == "Previous") {
+                    int j = i;
+                    PlaybackRepeatActions repeat_group = interface_action_service.FindActionGroup ("PlaybackRepeat")
+                         as PlaybackRepeatActions;
+                    if (repeat_group != null) {
+                        menu.Insert (repeat_group.CreateSubmenu (), i++ + 2);
+                    }
+                    
+                    PlaybackShuffleActions shuffle_group = interface_action_service.FindActionGroup ("PlaybackShuffle")
+                         as PlaybackShuffleActions;
+                    if (shuffle_group != null) {
+                        menu.Insert (shuffle_group.CreateSubmenu (), i++ + 2);
+                    }
+                    
+                    if (j != i) {
+                        menu.Insert (new SeparatorMenuItem (), i++ + 2);
+                    }
+                    
                     rating_menu_item = new RatingMenuItem ();
                     rating_menu_item.Activated += OnItemRatingActivated;
                     ToggleRatingMenuSensitive ();
@@ -139,7 +156,12 @@ namespace Banshee.NotificationArea
                 }
             }
 
-            ServiceManager.PlayerEngine.EventChanged += OnPlayerEngineEventChanged;
+            ServiceManager.PlayerEngine.ConnectEvent (OnPlayerEvent, 
+               PlayerEvent.Iterate |
+               PlayerEvent.StartOfStream |
+               PlayerEvent.EndOfStream |
+               PlayerEvent.TrackInfoUpdated |
+               PlayerEvent.StateChange);
             
             // Forcefully load this
             show_notifications = ShowNotifications;
@@ -160,13 +182,19 @@ namespace Banshee.NotificationArea
                 notif_area = null;
             }
             
-            ServiceManager.PlayerEngine.EventChanged -= OnPlayerEngineEventChanged;
+            ServiceManager.PlayerEngine.DisconnectEvent (OnPlayerEvent);
             
             elements_service.PrimaryWindowClose = null;
             
-            interface_action_service.UIManager.RemoveUi (ui_manager_id);
-            interface_action_service.UIManager.RemoveActionGroup (actions);
+            Gtk.Action close_action = interface_action_service.GlobalActions["CloseAction"];
+            if (close_action != null) {
+                interface_action_service.GlobalActions.Remove (close_action);
+            }
             
+            interface_action_service.RemoveActionGroup ("NotificationArea");
+            interface_action_service.UIManager.RemoveUi (ui_manager_id);
+            
+            actions = null;
             elements_service = null;
             interface_action_service = null;
             
@@ -203,6 +231,15 @@ namespace Banshee.NotificationArea
             return true;
         }
         
+        private void DisposeNotificationArea ()
+        {
+            if (notif_area != null) {
+                notif_area.Disconnected -= OnNotificationAreaDisconnected;
+                notif_area.Activated -= OnNotificationAreaActivated;
+                notif_area.PopupMenuEvent -= OnNotificationAreaPopupMenuEvent;
+            }
+        }
+        
         private void RegisterCloseHandler ()
         {
             if (elements_service.PrimaryWindowClose == null) {
@@ -226,12 +263,7 @@ namespace Banshee.NotificationArea
         private void OnNotificationAreaDisconnected (object o, EventArgs args)
         {
             // Ensure we don't keep the destroyed reference around
-            if (notif_area != null) {
-                notif_area.Disconnected -= OnNotificationAreaDisconnected;
-                notif_area.Activated -= OnNotificationAreaActivated;
-                notif_area.PopupMenuEvent -= OnNotificationAreaPopupMenuEvent;
-            }
-            
+            DisposeNotificationArea ();
             BuildNotificationArea ();
         }
         
@@ -276,27 +308,27 @@ namespace Banshee.NotificationArea
             ShowNotifications = ((ToggleAction)actions["ToggleNotificationsAction"]).Active;
         }
         
-        private void OnPlayerEngineEventChanged (object o, PlayerEngineEventArgs args) 
+        private void OnPlayerEvent (PlayerEventArgs args) 
         {
             switch (args.Event) {
-                case PlayerEngineEvent.Iterate:
+                case PlayerEvent.Iterate:
                     if (current_track != ServiceManager.PlayerEngine.CurrentTrack) {
                         current_track = ServiceManager.PlayerEngine.CurrentTrack;
                         ShowTrackNotification ();
                     }
                     break;
-                case PlayerEngineEvent.StartOfStream:
-                case PlayerEngineEvent.TrackInfoUpdated:
+                case PlayerEvent.StartOfStream:
+                case PlayerEvent.TrackInfoUpdated:
                     ToggleRatingMenuSensitive ();
                     ShowTrackNotification ();
                     break;
-                case PlayerEngineEvent.EndOfStream:
+                case PlayerEvent.EndOfStream:
                     current_track = null;
                     ToggleRatingMenuSensitive ();
                     break;
             }
             
-            notif_area.PlayerEngineEventChanged (args);
+            notif_area.OnPlayerEvent (args);
         }
         
         private void OnItemRatingActivated (object o, EventArgs args)
