@@ -49,10 +49,14 @@ namespace Banshee.PlayQueue
 {
     public class PlayQueueSource : PlaylistSource, IBasicPlaybackController, IDisposable
     {
-        private static string special_playlist_name = typeof (PlayQueueSource).ToString ();
+        private static string special_playlist_name = "Play Queue";//typeof (PlayQueueSource).ToString ();
 
         private DatabaseTrackInfo playing_track;
         private bool actions_loaded = false;
+
+        protected override bool HasArtistAlbum {
+            get { return false; }
+        }
         
         public PlayQueueSource () : base (Catalog.GetString ("Play Queue"), null, 0)
         {
@@ -64,11 +68,13 @@ namespace Banshee.PlayQueue
             
             ((DatabaseTrackListModel)TrackModel).ForcedSortQuery = "CorePlaylistEntries.EntryID ASC";
             
-            ServiceManager.PlayerEngine.EventChanged += OnPlayerEngineEventChanged;
+            ServiceManager.PlayerEngine.ConnectEvent (OnPlayerEvent);
             ServiceManager.PlaybackController.Transition += OnCanonicalPlaybackControllerTransition;
 
             ServiceManager.SourceManager.AddSource (this);
             
+            // TODO change this Gtk.Action code so that the actions can be removed.  And so this
+            // class doesn't depend on Gtk/ThickClient.
             InterfaceActionService uia_service = ServiceManager.Get<InterfaceActionService> ();
             uia_service.TrackActions.Add (new ActionEntry [] {
                 new ActionEntry ("AddToPlayQueueAction", Stock.Add,
@@ -106,19 +112,29 @@ namespace Banshee.PlayQueue
             ServiceManager.SourceManager.MusicLibrary.TracksDeleted += HandleTracksDeleted;
             ServiceManager.SourceManager.VideoLibrary.TracksChanged += HandleTracksChanged;
             ServiceManager.SourceManager.VideoLibrary.TracksDeleted += HandleTracksDeleted;
+            
+            TrackModel.Reloaded += delegate {
+                if (this == ServiceManager.PlaybackController.Source && Count == 0) {
+                    ServiceManager.PlaybackController.Source = (ITrackModelSource)ServiceManager.SourceManager.DefaultSource;
+                }
+            };
+            
+            Reload ();
 
-            SetAsPlayingSource ();
+            SetAsPlaybackSourceUnlessPlaying ();
         }
         
-        private void SetAsPlayingSource ()
+        private void SetAsPlaybackSourceUnlessPlaying ()
         {
-            if (Count > 0 && ServiceManager.PlayerEngine.CurrentState != PlayerEngineState.Playing) {
-                ServiceManager.PlaybackController.Source = this;
+            if (Count > 0) {
+                ServiceManager.PlaybackController.NextSource = this;
             }
         }
 
         public void Dispose ()
         {
+            ServiceManager.PlayerEngine.DisconnectEvent (OnPlayerEvent);
+
             if (ClearOnQuitSchema.Get ()) {
                 OnClearPlayQueue (this, EventArgs.Empty);
             }
@@ -143,7 +159,7 @@ namespace Banshee.PlayQueue
         protected override void OnTracksAdded ()
         {
             base.OnTracksAdded ();
-            SetAsPlayingSource ();
+            SetAsPlaybackSourceUnlessPlaying ();
         }
         
         protected override void OnUpdated ()
@@ -162,12 +178,16 @@ namespace Banshee.PlayQueue
             }
         }
         
-        private void OnPlayerEngineEventChanged (object o, PlayerEngineEventArgs args)
-        { 
-            if (args.Event == PlayerEngineEvent.EndOfStream) {
+        private void OnPlayerEvent (PlayerEventArgs args)
+        {
+            if (args.Event == PlayerEvent.EndOfStream) {
                 RemovePlayingTrack ();
-            } else if (args.Event == PlayerEngineEvent.StartOfStream) {
-                playing_track = ServiceManager.PlayerEngine.CurrentTrack as DatabaseTrackInfo; 
+            } else if (args.Event == PlayerEvent.StartOfStream) {
+                if (this == ServiceManager.PlaybackController.Source) {
+                    playing_track = ServiceManager.PlayerEngine.CurrentTrack as DatabaseTrackInfo; 
+                } else {
+                    playing_track = null;
+                }
             }
         }
         
@@ -178,6 +198,7 @@ namespace Banshee.PlayQueue
         
         private void OnClearPlayQueue (object o, EventArgs args)
         {
+            playing_track = null;
             RemoveTrackRange ((DatabaseTrackListModel)TrackModel, new Hyena.Collections.RangeCollection.Range (0, Count));
             Reload ();
         }
@@ -216,10 +237,8 @@ namespace Banshee.PlayQueue
         {
             RemovePlayingTrack ();
             
-            if (Count <= 0) {
-                playing_track = null;
-                ServiceManager.PlaybackController.Source = (ITrackModelSource)ServiceManager.SourceManager.DefaultSource;
-                ServiceManager.PlaybackController.Next ();
+            if (Count == 0) {
+                ServiceManager.PlaybackController.Next (restart);
                 return;
             }
             
