@@ -61,6 +61,8 @@ namespace Hyena.Data.Gui
         private CellContext cell_context;
         private Pango.Layout pango_layout;
         
+        private List<int> selected_rows = new List<int> ();
+        
         private Theme theme;
         protected Theme Theme {
             get { return theme; }
@@ -160,6 +162,9 @@ namespace Hyena.Data.Gui
         
         private void PaintHeaderCell (Rectangle area, Rectangle clip, int ci, bool dragging)
         {
+            if (ci < 0 || column_cache.Length <= ci)
+                return;
+
             if (dragging) {
                 Theme.DrawColumnHighlight (cairo_context, area, 
                     CairoExtensions.ColorShade (Theme.Colors.GetWidgetColor (GtkColorClass.Dark, StateType.Normal), 0.9));
@@ -175,7 +180,7 @@ namespace Hyena.Data.Gui
                 cairo_context.LineTo (area.Right - 0.5, area.Bottom);
                 cairo_context.Stroke ();
             }
-            
+
             ColumnCell cell = column_cache[ci].Column.HeaderCell;
             
             if (cell != null) {
@@ -183,6 +188,7 @@ namespace Hyena.Data.Gui
                 cairo_context.Translate (area.X, area.Y);
                 cell_context.Area = area;
                 cell_context.Clip = clip;
+                cell_context.Sensitive = true;
                 cell.Render (cell_context, StateType.Normal, area.Width, area.Height);
                 cairo_context.Restore ();
             }
@@ -310,7 +316,7 @@ namespace Hyena.Data.Gui
             
             int selection_height = 0;
             int selection_y = 0;
-            List<int> selected_rows = new List<int> ();
+            selected_rows.Clear ();
 
             for (int ri = first_row; ri < last_row; ri++) {
                 if (Selection.Contains (ri)) {
@@ -405,17 +411,17 @@ namespace Hyena.Data.Gui
                 
                 cell_area.Width = column_cache[ci].Width;
                 cell_area.X = column_cache[ci].X1 + area.X;
-                PaintCell (item, ci, row_index, cell_area, sensitive ? state : StateType.Insensitive, false);
+                PaintCell (item, ci, row_index, cell_area, sensitive, state, false);
             }
             
             if (pressed_column_is_dragging && pressed_column_index >= 0) {
                 cell_area.Width = column_cache[pressed_column_index].Width;
                 cell_area.X = pressed_column_x_drag - list_interaction_alloc.X;
-                PaintCell (item, pressed_column_index, row_index, cell_area, state, true);
+                PaintCell (item, pressed_column_index, row_index, cell_area, sensitive, state, true);
             }
         }
         
-        private void PaintCell (object item, int column_index, int row_index, Rectangle area, 
+        private void PaintCell (object item, int column_index, int row_index, Rectangle area, bool sensitive,
             StateType state, bool dragging)
         {
             ColumnCell cell = column_cache[column_index].Column.GetCell (0);
@@ -432,6 +438,7 @@ namespace Hyena.Data.Gui
             cairo_context.Save ();
             cairo_context.Translate (area.X, area.Y);
             cell_context.Area = area;
+            cell_context.Sensitive = sensitive;
             cell.Render (cell_context, dragging ? StateType.Normal : state, area.Width, area.Height);
             cairo_context.Restore ();
         }
