@@ -34,7 +34,7 @@ using Hyena.Gui.Theming;
 
 namespace Hyena.Data.Gui
 {
-    public partial class ListView<T> : Container
+    public partial class ListView<T> : Widget
     {
         private Rectangle list_rendering_alloc;
         private Rectangle header_rendering_alloc;
@@ -52,8 +52,6 @@ namespace Hyena.Data.Gui
             WidgetFlags |= WidgetFlags.Realized | WidgetFlags.NoWindow;
             
             GdkWindow = Parent.GdkWindow;
-            theme = new GtkTheme (this);
-            //graphics.RefreshColors ();
             
             WindowAttr attributes = new WindowAttr ();
             attributes.WindowType = Gdk.WindowType.Child;
@@ -63,7 +61,6 @@ namespace Hyena.Data.Gui
             attributes.Height = Allocation.Height;
             attributes.Wclass = WindowClass.InputOnly;
             attributes.EventMask = (int)(
-                GdkWindow.Events |
                 EventMask.PointerMotionMask |
                 EventMask.KeyPressMask |
                 EventMask.KeyReleaseMask |
@@ -80,11 +77,14 @@ namespace Hyena.Data.Gui
             
             OnDragSourceSet ();
             MoveResize (Allocation);
+            RegenerateCanvases ();
+            
+            base.OnRealized ();
         }
         
         protected override void OnUnrealized ()
         {
-            WidgetFlags ^= WidgetFlags.Realized;
+            WidgetFlags &= ~WidgetFlags.Realized;
             
             event_window.UserData = IntPtr.Zero;
             event_window.Destroy ();
@@ -103,7 +103,7 @@ namespace Hyena.Data.Gui
         
         protected override void OnUnmapped ()
         {
-            WidgetFlags ^= WidgetFlags.Mapped;
+            WidgetFlags &= ~WidgetFlags.Mapped;
             event_window.Hide ();
         }
         
@@ -131,7 +131,12 @@ namespace Hyena.Data.Gui
             list_interaction_alloc = list_rendering_alloc;
             list_interaction_alloc.Offset (-allocation.X, -allocation.Y);
             
-            int canvas_width = Math.Max (1, list_rendering_alloc.Width);
+            header_width = header_interaction_alloc.Width;
+        }
+        
+        private void RegenerateCanvases ()
+        {
+            int canvas_width = Math.Max (1, header_width);
             int canvas_height = Math.Max (1, RowsInView * RowHeight);
             
             if (canvas_alloc.Width != canvas_width || canvas_alloc.Height != canvas_height) {
@@ -159,8 +164,8 @@ namespace Hyena.Data.Gui
             if (Theme == null) {
                 return;
             }
-            requisition.Width = Theme.InnerBorderWidth * 2;
-            requisition.Height = HeaderHeight;
+            requisition.Width = Theme.TotalBorderWidth * 2;
+            requisition.Height = HeaderHeight + Theme.TotalBorderWidth * 2;
         }
         
         protected override void OnSizeAllocated (Rectangle allocation)
@@ -172,11 +177,16 @@ namespace Hyena.Data.Gui
             }
             
             MoveResize (allocation);
+            RecalculateColumnSizes ();
+            RegenerateColumnCache ();
+            RegenerateCanvases ();
            
             if (vadjustment != null) {
+                hadjustment.PageSize = header_interaction_alloc.Width;
+                hadjustment.PageIncrement = header_interaction_alloc.Width;
                 vadjustment.PageSize = list_rendering_alloc.Height;
                 vadjustment.PageIncrement = list_rendering_alloc.Height;
-                UpdateAdjustments (null, null);
+                UpdateAdjustments ();
             }
             
             ICareAboutView model = Model as ICareAboutView;
@@ -184,7 +194,6 @@ namespace Hyena.Data.Gui
                 model.RowsInView = RowsInView;
             }
             
-            RegenerateColumnCache ();
             InvalidateList (false);
         }
         
