@@ -27,6 +27,7 @@
 //
 
 using System;
+using System.Text;
 using System.Collections.Generic;
 
 namespace Hyena
@@ -100,7 +101,7 @@ namespace Hyena
             set { debugging = value; }
         }
         
-        private static void Commit (LogEntryType type, string message, string details, bool showUser)
+        public static void Commit (LogEntryType type, string message, string details, bool showUser)
         {
             if (type == LogEntryType.Debug && !Debugging) {
                 return;
@@ -318,7 +319,7 @@ namespace Hyena
         
         public static void Warning (string message, string details)
         {
-            Warning (message, details, true);
+            Warning (message, details, false);
         }
         
         public static void Warning (string message, string details, bool showUser)
@@ -342,7 +343,7 @@ namespace Hyena
         
         public static void Error (string message, string details)
         {
-            Error (message, details, true);
+            Error (message, details, false);
         }
         
         public static void Error (string message, string details, bool showUser)
@@ -354,6 +355,11 @@ namespace Hyena
         {
             Error (message, null, showUser);
         }
+
+        public static void ErrorFormat (string format, params object [] args)
+        {
+            Error (String.Format (format, args));
+        }
         
         #endregion
         
@@ -361,8 +367,30 @@ namespace Hyena
         
         public static void Exception (Exception e)
         {
+            Exception (null, e);
+        }
+        
+        public static void Exception (string message, Exception e)
+        {
+            Stack<Exception> exception_chain = new Stack<Exception> ();
+            StringBuilder builder = new StringBuilder ();
+            
+            while (e != null) {
+                exception_chain.Push (e);
+                e = e.InnerException;
+            }
+            
+            while (exception_chain.Count > 0) {
+                e = exception_chain.Pop ();
+                builder.AppendFormat ("{0} (in `{1}')", e.Message, e.Source).AppendLine ();
+                builder.Append (e.StackTrace);
+                if (exception_chain.Count > 0) {
+                    builder.AppendLine ();
+                }
+            }
+        
             // FIXME: We should save these to an actual log file
-            Console.WriteLine (e);
+            Log.Warning (message ?? "Caught an exception", builder.ToString (), false);
         }
         
         #endregion
